// Copyright (C) 2008
// by all contributors.

// This file is part of SCNSL.

// SCNSL is free software:
// you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option)
// any later version.

// SCNSL is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with SCNSL,
// in a file named LICENSE.txt.
// If not, see <http://www.gnu.org/licenses/>.



/// @file
/// A plugin manager.

#include <cstdlib>
#include <dlfcn.h>

#include "Extension_if_t.hh"
#include "PluginManager_t.hh"


using namespace Scnsl::Setup;


const char * const PluginManager_t::_SYMBOL = "get_extensions";


PluginManager_t::PluginManager_t()
    throw():
    _handlers()
{
    // Nohting to do.
}


PluginManager_t::~PluginManager_t()
    throw()
{
    // Reverse order.
    for ( HandlerList_t::reverse_iterator i = _handlers.rbegin();
        i != _handlers.rend();
        ++i )
    {
        dlclose( *i );
    }
}


void PluginManager_t::load( const PluginName_t & name, ExtensionList_t & extensionList )
    throw ( std::domain_error )
{
    lib_handler_t handle = dlopen( name.c_str(), RTLD_LAZY | RTLD_LOCAL );

    if ( handle == NULL )
    {
        throw std::domain_error( dlerror() );
    }

    // Cleaning up possible old error codes.
    dlerror();
    register void * value = dlsym( handle, _SYMBOL );
    char * err = dlerror();
    if ( err != NULL )
    {
        throw std::logic_error( err );
    }

    union avoid_cast_warning_t
    {
        void * v;
        get_extensions_t load;
    } u;

    u.v = value;

    ExtensionList_t ext;
    (*u.load)( & ext );

    // Checking if an extension is already loaded:
    for ( ExtensionList_t::iterator i = ext.begin();
        i != ext.end();
        ++ i )
    {
        ExtensionList_t::iterator f = extensionList.find( (*i).second->getId() );
        if (f != extensionList.end() )
        {
            throw std::domain_error( "Extension ID already in use." );
        }
    }

    // Finally, we can update loader status and extension list.
    // Exception neutrality reached. :)

    _handlers.push_back( handle );

    for ( ExtensionList_t::iterator i = ext.begin();
        i != ext.end();
        ++ i )
    {
        extensionList[ (*i).second->getId() ] = (*i).second;
    }
}
