#include "DefaultEnvironment_t.hh"

using Scnsl::Utils::DefaultEnvironment_t;


DefaultEnvironment_t::DefaultEnvironment_t(
    const double alpha, const propagation_t propagation)
    :
    _ALPHA(alpha),
    _PROPAGATION(propagation)
{
    // Nothing to do.
}

DefaultEnvironment_t::~DefaultEnvironment_t()
{
    // Nothing to do.
}

void DefaultEnvironment_t::createInstance(const double alpha,const propagation_t propagation)
{
    static DefaultEnvironment_t p(alpha, propagation);

    DefaultEnvironment_t::_env = &p;

}


// ////////////////////////////////////////////////////////////////
// Interface methods.
// ////////////////////////////////////////////////////////////////


double DefaultEnvironment_t::getReceiverPower( const node_properties_t & sp,
    const node_properties_t & rp){

    const position_t dx = sp.x - rp.x;
    const position_t dy = sp.y - rp.y;
    const position_t dz = sp.z - rp.z;
    const position_t d2 = dx*dx + dy*dy + dz*dz;
    const double distance = sqrt( d2 );

    return (sp.transmission_power / pow( distance, _ALPHA ));

}

double DefaultEnvironment_t::getMaxDelay( const node_properties_t & sp,
    const node_properties_t & rp)
{
    return pow(sp.transmission_power/rp.receiving_threshold, 1/_ALPHA)/_PROPAGATION;

}

double DefaultEnvironment_t::getBitErrorRate(const node_properties_t & sp,
    const node_properties_t & rp)
{
    return ((sp.x+sp.y+sp.z)*0.03 + (rp.x+rp.y+rp.z)*0.01);
}

Scnsl::Core::propagation_t DefaultEnvironment_t::getPropagation(){
    return _PROPAGATION;
}




