
#include <sstream>
#include <systemc>
#include <tlm.h>
#include <exception>

#include <scnsl.hh>
#include "MyTask_t.hh"



using namespace Scnsl::Setup;
using namespace Scnsl::BuiltinPlugin;


int sc_main( int argc, char * argv[] )
{
	try {

        // Which testcase? (from 1 to 3)
        unsigned int TESTCASE = 0;
        if ( argc == 2 )
        {
            std::stringstream ss;
            ss << argv[ 1 ];
            ss >> TESTCASE;
        }
        if ( TESTCASE > 4 || TESTCASE < 1 )
        {
            TESTCASE = 1;
        }

        // Singleton.
        Scnsl::Setup::Scnsl_t * sim = Scnsl::Setup::Scnsl_t::get_instance();

        // Nodes creation:
        Scnsl::Core::Node_t * n0 = sim->createNode();
        Scnsl::Core::Node_t * n1 = sim->createNode();

        CoreChannelSetup_t ccs;

        ccs.extensionId = "core";
        ccs.channel_type = CoreChannelSetup_t::DELAYED_SHARED;
        ccs.name = "DelayedSharedChannel";
        ccs.alpha = 1;
        ccs.propagation = CoreChannelSetup_t::WATER_SOUND_SPEED;
        ccs.nodes_number = 2;
    
        Scnsl::Utils::DefaultEnvironment_t::createInstance(ccs.alpha, 1480);
        
        Scnsl::Core::Channel_if_t * ch = sim->createChannel( ccs );

        const bool IS_SENDER = true;
        const Scnsl::Core::task_id_t id0 = 0;
        const Scnsl::Core::task_id_t id1 = 1;
        const Scnsl::Core::size_t PROXIES = 1;

        MyTask_t t0( "Task0", IS_SENDER, TESTCASE, id0, n0, PROXIES );
        MyTask_t t1( "Task1", ! IS_SENDER, TESTCASE, id1, n1, PROXIES );

        // Adding tracing features:
        CoreTracingSetup_t cts;
        cts.extensionId = "core";
        // - Setting the formatter:
        cts.formatterExtensionId = "core";
        cts.formatterName = "basic";
        // Setting the filter:
        cts.filterExtensionId = "core";
        cts.filterName = "basic";
        // Core formatter specific option:
        // printing also the type of trace:
        cts.print_trace_type = true;

        // - Setting to trace only user-like infos:
        cts.info = 5;
        cts.debug = 0;
        cts.log = 5;
        cts.error = 0;
        cts.warning = 0;
        cts.fatal = 0;
        // - Creating:
        Scnsl_t::Tracer_t * tracer1 = sim->createTracer( cts );
        // - Setting the output stream:
        tracer1->addOutput( & std::cout );
        // - Adding to trace:
        tracer1->trace( & t0 );
        tracer1->trace( & t1 );
        tracer1->trace( ch );

        // - Setting to trace backend-like infos:
        cts.info = 0;
        cts.debug = 5;
        cts.log = 0;
        cts.error = 5;
        cts.warning = 5;
        cts.fatal = 5;
        // - Creating:
        Scnsl_t::Tracer_t * tracer2 = sim->createTracer( cts );
        // - Setting the output stream:
        tracer2->addOutput( & std::cerr );
        // - Adding to trace:
        tracer2->trace( & t0 );
        tracer2->trace( & t1 );
        tracer2->trace( ch );
        tracer2->trace( Scnsl::Utils::EventsQueue_t::get_instance() );

        // Binding:
        BindSetup_base_t bsb0;
        bsb0.extensionId = "core";
        bsb0.destinationNode = n1;
        bsb0.node_binding.x = 0;
        bsb0.node_binding.y = 0;
        bsb0.node_binding.z = 0;
        bsb0.node_binding.bitrate = 100;
        bsb0.node_binding.transmission_power = 1000;
        bsb0.node_binding.receiving_threshold = 1;

        sim->bind( n0, ch, bsb0 );
        sim->bind( & t0, & t1, ch, bsb0, NULL );

        // A reference to the instance of the events queue.
        Scnsl::Utils::EventsQueue_t * eventsQueue = Scnsl::Utils::EventsQueue_t::get_instance();

        if ( TESTCASE == 1 || TESTCASE == 2 )
        {
            BindSetup_base_t bsb1;
            bsb1.extensionId = "core";
            bsb1.destinationNode = NULL;
            bsb1.node_binding.x = 800;
            bsb1.node_binding.y = 0;
            bsb1.node_binding.z = 0;
            bsb1.node_binding.bitrate = 100;
            bsb1.node_binding.transmission_power = 1000;
            bsb1.node_binding.receiving_threshold = 1;

            sim->bind( n1, ch, bsb1 );
            sim->bind( & t1, & t0, ch, bsb1, NULL );

            Scnsl::Core::node_properties_t np1;
            np1.x = 1500;
            np1.y = 0;
            np1.z = 0;
            np1.bitrate = 100;
            np1.transmission_power = 1000;
            np1.receiving_threshold = 1;

            // Registering callback:
            eventsQueue->registerNodeProperties( sc_core::sc_time( 0.7, sc_core::SC_SEC ), n1, np1, ch );
        }
        else if ( TESTCASE == 3 )
        {
            BindSetup_base_t bsb1;
            bsb1.extensionId = "core";
            bsb1.destinationNode = NULL;
            bsb1.node_binding.x = 1500;
            bsb1.node_binding.y = 0;
            bsb1.node_binding.z = 0;
            bsb1.node_binding.bitrate = 100;
            bsb1.node_binding.transmission_power = 1000;
            bsb1.node_binding.receiving_threshold = 1;

            sim->bind( n1, ch, bsb1 );
            sim->bind( & t1, & t0, ch, bsb1, NULL );

            Scnsl::Core::node_properties_t np1;
            np1.x = 500;
            np1.y = 0;
            np1.z = 0;
            np1.bitrate = 100;
            np1.transmission_power = 1000;
            np1.receiving_threshold = 1;

            // Registering callback:
            eventsQueue->registerNodeProperties( sc_core::sc_time( 0.7, sc_core::SC_SEC ), n1, np1, ch );
        }
        else
        {
            BindSetup_base_t bsb1;
            bsb1.extensionId = "core";
            bsb1.destinationNode = NULL;
            bsb1.node_binding.x = 800;
            bsb1.node_binding.y = 0;
            bsb1.node_binding.z = 0;
            bsb1.node_binding.bitrate = 100;
            bsb1.node_binding.transmission_power = 1000;
            bsb1.node_binding.receiving_threshold = 1;

            sim->bind( n1, ch, bsb1 );
            sim->bind( & t1, & t0, ch, bsb1, NULL );

            Scnsl::Core::node_properties_t np1;
            np1.x = 1500;
            np1.y = 0;
            np1.z = 0;
            np1.bitrate = 100;
            np1.transmission_power = 1000;
            np1.receiving_threshold = 1;

            // Registering callback:
            eventsQueue->registerNodeProperties( sc_core::sc_time( 0.7, sc_core::SC_SEC ), n1, np1, ch );

            Scnsl::Core::node_properties_t np2;
            np2.x = 800;
            np2.y = 0;
            np2.z = 0;
            np2.bitrate = 100;
            np2.transmission_power = 1000;
            np2.receiving_threshold = 1;

            // Registering callback:
            eventsQueue->registerNodeProperties( sc_core::sc_time( 0.71, sc_core::SC_SEC ), n1, np2, ch );
        }

        sc_core::sc_start( sc_core::sc_time( 2, sc_core::SC_SEC ) );
        sc_core::sc_stop();

	}
    catch ( std::exception & e)
    {
        std::cerr << "Exception in sc_main(): " << e.what() << std::endl;
        return 1;
	}
	return 0;
}
